using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;

using System.Threading;
using System.Net;
using System.Net.Sockets;
using AddiData.ADDIControls;
using AddiData.ADDISocket;
using AddiData;

namespace Sequence
{
    public partial class Sequence : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXExxxx_WebReference.MSXExxxx MSXExxxx;
        private MXCommon_WebReference.DefaultResponse g_MXResponse;
        private MSXExxxx_WebReference.DefaultResponse g_Response;
        private CheckBox[] g_DataFrame = new CheckBox[5];
        private ComboBox[] g_SequenceChannels = new ComboBox[16];

        /// <summary>
        /// Store the type of the system
        /// </summary>
        private string systemType;

        /// <summary>
        /// True if the system is a MSX-E3311
        /// </summary>
        private bool isMsxe3311;
        
        /// <summary>
        /// Store the types of the channels
        /// </summary>
        private int[] channelType;

        /// <summary>
        /// Store the number of acquisition channels
        /// </summary>
        private ulong acquisitionChannelsCount = 0;

        /// <summary>
        /// Store a string with the unity of the channel
        /// </summary>
        private string[] channelUnity = new string[16];

        /// <summary>
        /// Store the possible trigger mask (or source)
        /// </summary>
        ArrayList possibleTriggerMask;

        ArrayList g_alAcquisitionChannels;

        /// <summary>
        /// Displayed text
        /// </summary>
        String displayedText = string.Empty;

        private int g_RetValue = 0;
        private int g_syserrno = 0;
        Socket SocketClient = null;
        private int g_iDataStreamSize = 0;
        AsyncClientStream g_Client = null;
        MSXExxxx_WebReference.AcquisitionSequenceInitAndStartChannelListParam g_SequenceChannelList;

        public Sequence()
        {
            InitializeComponent();
        }

        public static String ConvertFromTime_T(byte[] bBuffer, int iIndex)
        {
            ulong ulTimeStampHigh = BitConverter.ToUInt32(bBuffer, iIndex);
            ulong ulTimeStampLow = BitConverter.ToUInt32(bBuffer, iIndex + 4);

            return (ConvertFromTime_T(ulTimeStampHigh, ulTimeStampLow));
        }

        public static DateTime ConvertFromTime_T(long time_t)
        {
            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (time_t * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return netTimeT;
        }

        public static String ConvertFromTime_T(ulong ulTimeStampHigh, ulong ulTimeStampLow)
        {
            ulong ulTimeStampLowms = (ulong)(ulTimeStampLow / 1000);
            ulong ulTimeStampLowmus = (ulong)(ulTimeStampLow % 1000);

            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (long)(ulTimeStampHigh * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return (netTimeT.ToString() + "." + ulTimeStampLowms.ToString("D3") + "." + ulTimeStampLowmus.ToString("D3"));
        }

        /// <summary>
        /// Create the list of possible trigger mask, and update the combo box
        /// </summary>
        /// <param name="hasCompare"></param>
        /// <param name="hasIndex"></param>
        private void CreatePossibleTriggerMask(bool hasCompare, bool hasIndex)
        {
            LB_TriggerMask.DataSource = null;

            // Trigger source list box initialisation
            possibleTriggerMask = new ArrayList();
            possibleTriggerMask.Add(new ListData("Disabled", 0));
            possibleTriggerMask.Add(new ListData("Hardware trigger", 1));
            possibleTriggerMask.Add(new ListData("Synchro trigger", 2));

            if (hasCompare == true)
                possibleTriggerMask.Add(new ListData("Compare trigger", 4));

            if (hasIndex == true)
                possibleTriggerMask.Add(new ListData("Index trigger", 8));

            LB_TriggerMask.DisplayMember = "Item";
            LB_TriggerMask.ValueMember = "Value";
            LB_TriggerMask.DataSource = possibleTriggerMask;
        }


        private void Sequence_Load(object sender, EventArgs e)
        {
            // Load parameter
            this.ipTextBox.Text = Properties.Settings.Default.ip;


            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXExxxx = new MSXExxxx_WebReference.MSXExxxx();
            MX_Common.Timeout = 5000;
            MSXExxxx.Timeout = 5000;

            // Refresh time unit list box initialisation
            ArrayList RefreshTimeUnits = new ArrayList();
            RefreshTimeUnits.Add(new ListData("us", 0));
            RefreshTimeUnits.Add(new ListData("ms", 1));
            RefreshTimeUnits.Add(new ListData("s", 2));

            LB_RefreshTimeUnit.DisplayMember = "Item";
            LB_RefreshTimeUnit.ValueMember = "Value";
            LB_RefreshTimeUnit.DataSource = RefreshTimeUnits;

            this.CreatePossibleTriggerMask(false, false);

            // Trigger mode list box initialisation
            ArrayList TriggerModes = new ArrayList();
            TriggerModes.Add(new ListData("One shot trigger", 1));
            TriggerModes.Add(new ListData("Sequence trigger", 2));

            LB_TriggerMode.DisplayMember = "Item";
            LB_TriggerMode.ValueMember = "Value";
            LB_TriggerMode.DataSource = TriggerModes;


            // HW Trigger edge list box initialisation
            ArrayList HWTriggerEdges = new ArrayList();
            HWTriggerEdges.Add(new ListData("Rising edge", 1));
            HWTriggerEdges.Add(new ListData("Falling edge", 2));
            HWTriggerEdges.Add(new ListData("Both front", 3));

            LB_HWEdge.DisplayMember = "Item";
            LB_HWEdge.ValueMember = "Value";
            LB_HWEdge.DataSource = HWTriggerEdges;


            g_SequenceChannels[0] = LB_Sequence0;
            g_SequenceChannels[1] = LB_Sequence1;
            g_SequenceChannels[2] = LB_Sequence2;
            g_SequenceChannels[3] = LB_Sequence3;
            g_SequenceChannels[4] = LB_Sequence4;
            g_SequenceChannels[5] = LB_Sequence5;
            g_SequenceChannels[6] = LB_Sequence6;
            g_SequenceChannels[7] = LB_Sequence7;
            g_SequenceChannels[8] = LB_Sequence8;
            g_SequenceChannels[9] = LB_Sequence9;
            g_SequenceChannels[10] = LB_Sequence10;
            g_SequenceChannels[11] = LB_Sequence11;
            g_SequenceChannels[12] = LB_Sequence12;
            g_SequenceChannels[13] = LB_Sequence13;
            g_SequenceChannels[14] = LB_Sequence14;
            g_SequenceChannels[15] = LB_Sequence15;
            acquisitionChannelsCount = 16;

            for (long i = 0; i < 16; i++)
            {
                // Sequence channels list box initialisation
                ArrayList Sequences = new ArrayList();
                Sequences.Add(new ListData("Channel " + i.ToString(), i));

                g_SequenceChannels[i].DisplayMember = "Item";
                g_SequenceChannels[i].ValueMember = "Value";
                g_SequenceChannels[i].DataSource = Sequences;
                g_SequenceChannels[i].Enabled = false;
            }


            // Save the data frame check box
            g_DataFrame[0] = CB_DataFrameAbsoluteTimeStamp;
            g_DataFrame[1] = CB_DataFrameRelativeTimeStamp;
            g_DataFrame[2] = CB_DataFrameSequenceCounter;
            g_DataFrame[3] = CB_DataFrameStatus;
            g_DataFrame[4] = CB_DataFrameAnalog;

            GB_SequenceConfiguration.Enabled = false;

        }

        /// <summary>
        /// User clicks on the connect button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];
            ulong ulType = 0;
            ulong ulHWPosition = 0;
            ulong ulChannelIndex = 0;
            ulong ulAnaInptChannel = 0;
            ulong [] ulAnaInptType;
            ulong [] ulAnaInptICP;
            IPAddress address;
            bool hasCompare = false;
            bool hasIndex = false;

            if (IPAddress.TryParse(this.ipTextBox.Text, out address) == false)
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
    
            MX_Common.Url = "http://" + this.ipTextBox.Text + ":5555";
            MSXExxxx.Url = MX_Common.Url;

            // Get the module type
            try
            {
                g_MXResponse = MX_Common.GetModuleType(out pcName);
                if (g_MXResponse.iReturnValue != 0)
                {
                    MessageBox.Show("GetModuleType Error : " + g_MXResponse.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("GetModuleType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // Convert the system type into a string
            this.systemType = enc.GetString(pcName);
            if (systemType.Contains("MX331X{"))
            {
                this.isMsxe3311 = true;
            }
            else
            {
                this.isMsxe3311 = false;
            }

            // Get the number of acquisition channels
            try
            {
                g_Response = MSXExxxx.AcquisitionGetNumberOfChannels(0, out acquisitionChannelsCount);
                if (g_Response.iReturnValue != 0)
                {
                    MessageBox.Show("AcquisitionGetNumberOfChannels Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("AcquisitionGetNumberOfChannels Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // Get the types of the channels
            this.channelType = new int[acquisitionChannelsCount];
            for (ulong i = 0; i < acquisitionChannelsCount; i++)
            {
                try
                {
                    g_Response = MSXExxxx.AcquisitionGetChannelInfo(i, 0, out ulType, out ulHWPosition, out ulChannelIndex);
                    if (g_Response.iReturnValue != 0)
                    {
                        MessageBox.Show("AcquisitionGetChannelInfo Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                    // Store the type
                    channelType[i] = (int)ulType;

                    // Check if incremental counter or sin cos
                    if ((ulType == 7) || (ulType == 8))
                        hasCompare = true;

                    // Check if sin cos
                    if (ulType == 8)
                        hasIndex = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("AcquisitionGetChannelInfo Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

                {
                    // Sequence channels list box initialisation
                    ArrayList Sequences = new ArrayList();
                    for (long j = -1; j < (long)acquisitionChannelsCount; j++)
                    {
                        if (j != -1)
                            Sequences.Add(new ListData("Channel " + j.ToString(), j));
                        else
                            Sequences.Add(new ListData("", j));
                    }
                    g_SequenceChannels[i].DataSource = Sequences;
                    g_SequenceChannels[i].Visible = Convert.ToBoolean((long)i < (long)acquisitionChannelsCount);
                    g_SequenceChannels[i].SelectedIndex = (int)i + 1;
                    g_SequenceChannels[i].Enabled = true;
                }

            }

            for (ulong i = 0; i < acquisitionChannelsCount; i++)
            {
                switch (channelType[i])
                {
                    case 1:
                        channelUnity[i] = "C";
                        break;

                    case 3:
                        channelUnity[i] = "mm";
                        break;

                    case 4:
                    case 5:
                        try
                        {
                            g_Response = MSXExxxx.AnalogInputGetChannelsType(0, out ulAnaInptType, out ulAnaInptICP);
                            if (g_Response.iReturnValue != 0)
                            {
                                MessageBox.Show("AnalogInputGetChannelsType Error : " + g_Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                return;
                            }

                            if (ulAnaInptType[ulAnaInptChannel] == 0)
                            {
                                channelUnity[i] = "V";
                            }
                            else
                            {
                                channelUnity[i] = "A";
                            }
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show("AnalogInputGetChannelsType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            return;
                        }
                        ulAnaInptChannel++;
                        break;

                    default:
                        channelUnity[i] = "";
                        break;
                }
            }

            // Update the list of possible triggers
            CreatePossibleTriggerMask(hasCompare, hasIndex);

            L_SequenceSize.Text = "Sequence size:" + acquisitionChannelsCount.ToString();

            GB_SequenceConfiguration.Enabled = true;
            B_StartSample.Enabled = true;
        }


        private bool boCheckConfiguration(out MSXExxxx_WebReference.AcquisitionSequenceInitAndStartChannelListParam ulSequenceChannelList, out ulong ulSequenceSize, out ulong ulDataFormat, out int iDataStreamSize)
        {
            ulDataFormat = 0;
            iDataStreamSize = 0;
            ulSequenceSize = 0;

            // Check the channel sequence
            g_alAcquisitionChannels = new ArrayList();
            ulSequenceChannelList = new global::Sequence.MSXExxxx_WebReference.AcquisitionSequenceInitAndStartChannelListParam();
            ulSequenceChannelList.ulChannelList = new UInt64[16];

            for (int i = 0; i < 16; i++)
            {
                if (g_SequenceChannels[i].Enabled == true)
                {
                    if (Convert.ToInt32(g_SequenceChannels[i].SelectedValue) != -1)
                    {
                        g_alAcquisitionChannels.Add("Channel " + g_SequenceChannels[i].SelectedValue.ToString() + ":");
                        ulSequenceChannelList.ulChannelList[ulSequenceSize] = Convert.ToUInt64(g_SequenceChannels[i].SelectedValue);
                        ulSequenceSize = ulSequenceSize + 1;
                    }
                }
            }

            if (ulSequenceSize == 0)
            {
                MessageBox.Show("No channel selected", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the number of sequence
            if ((Convert.ToInt64(TB_SequenceNbr.Text) < 0) || (Convert.ToInt64(TB_SequenceNbr.Text) > 0xFFFFFFFF))
            {
                MessageBox.Show("Sequence number value selection error (0 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the send data size
            if ((Convert.ToInt64(TB_TransferNbr.Text) < 1) || ((Convert.ToInt64(TB_TransferNbr.Text) > 0xFFFFFFFF) && (Convert.ToInt64(TB_SequenceNbr.Text) == 0)) || ((Convert.ToInt64(TB_TransferNbr.Text) > Convert.ToInt64(TB_SequenceNbr.Text)) && (Convert.ToInt64(TB_SequenceNbr.Text) != 0)))
            {
                if (Convert.ToInt64(TB_SequenceNbr.Text) == 0)
                {
                    MessageBox.Show("Send sequence number value selection error (1 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    MessageBox.Show("Send sequence number value selection error (1 <= value <= " + TB_SequenceNbr.Text + ")", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                return (false);
            }


            // Check the hardware trigger sequence number
            if ((TB_SequenceTrigger.Enabled == true) && ((Convert.ToInt64(TB_SequenceTrigger.Text) < 1) || (Convert.ToInt64(TB_SequenceTrigger.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Selection of sequence number to acquire for each trigger is wrong (0 < value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Get the data frame format
            for (int i = 0; i < g_DataFrame.Length; i++)
            {
                ulDataFormat = ulDataFormat | (Convert.ToUInt32(g_DataFrame[i].Checked) << i);
            }

            // Get the data strem size
            iDataStreamSize = (int)((ulSequenceSize * 4) +
                                    ((Convert.ToUInt32(CB_DataFrameAbsoluteTimeStamp.Checked) | (Convert.ToUInt32(CB_DataFrameRelativeTimeStamp.Checked))) * 8) +
                                    (Convert.ToUInt32(CB_DataFrameSequenceCounter.Checked) * 4) +
                                    (Convert.ToUInt32(CB_DataFrameStatus.Checked) * 4));

            return (true);
        }

        /// <summary>
        /// Client thread
        /// </summary>
        private void ClientThreadFct()
        {
            int SocketDataIndex;
            byte[] SocketData = new byte[g_Client.SingleFramesSize];

            while ((SocketData = g_Client.ReadData()) != null)
            {
                SocketDataIndex = 0;

                while (SocketData.Length != SocketDataIndex)
                {
                    String sOutString = string.Empty;

                    // Receive the time stamp?
                    if (CB_DataFrameAbsoluteTimeStamp.Checked == true)
                    {
                        sOutString = "Time stamp: " + ConvertFromTime_T(SocketData, SocketDataIndex) + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 8;
                    }
                    if (CB_DataFrameRelativeTimeStamp.Checked == true)
                    {
                        sOutString = "Time stamp: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + ":" + BitConverter.ToUInt32(SocketData, SocketDataIndex + 4).ToString() + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 8;
                    }

                    // Receive the sequence counter ?
                    if (CB_DataFrameSequenceCounter.Checked == true)
                    {
                        sOutString += "Sequence counter: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 4;
                    }

                    // Receive the status ?
                    if (CB_DataFrameStatus.Checked == true)
                    {
                        sOutString += "Status: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X8") + " Hex" +Environment.NewLine;
                        SocketDataIndex = SocketDataIndex + 4;
                    }

                    // Get the values of the channels of the acquisition
                    int iChannelsCpt = 0;
                    foreach (String Channel in g_alAcquisitionChannels)
                    {
                        // Test if user selected analog value
                        if (CB_DataFrameAnalog.Checked == true)
                        {
                            switch (this.channelType[g_SequenceChannelList.ulChannelList[iChannelsCpt]])
                            {
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                case 4:
                                case 5:
                                case 8:
                                    // 0 -> 5, 8 support float conversion
                                    sOutString += Channel + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString("F5") + channelUnity[g_SequenceChannelList.ulChannelList[iChannelsCpt]] + " ";
                                    break;
                                case 6:
                                case 7:
                                    // 6 (digital I/O) and 7 (incremental counter) do not supports float conversion
                                    sOutString += Channel + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X8") + " Hex" + " ";
                                    break;
                            }
                        }
                        // User selected digital value
                        else
                        {
                            // MSX-E3311: digital value, other MSX-E float value
                            if ((this.channelType[g_SequenceChannelList.ulChannelList[iChannelsCpt]] == 2) && (this.isMsxe3311 == false))
                            {
                                sOutString += Channel + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString("F5") + channelUnity[g_SequenceChannelList.ulChannelList[iChannelsCpt]] + " ";
                            }
                            else
                            {
                                sOutString += Channel + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X8") + " Hex" + " ";
                            }
                        }

                        SocketDataIndex = SocketDataIndex + 4;
                        iChannelsCpt++;
                        sOutString += Environment.NewLine;
                    }
                    
                    displayedText = sOutString;
                }
            }
        }

        private void B_StartSample_Click(object sender, EventArgs e)
        {
            ulong ulDataFormat;
            ulong ulSequenceSize;
            Thread ClientThread;

            // Check the user configuration
            if (boCheckConfiguration(out g_SequenceChannelList, out ulSequenceSize, out ulDataFormat, out g_iDataStreamSize) == true)
            {
                try
                {
                    // Create the client stream
                    g_Client = new AddiData.ADDISocket.AsyncClientStream();

                    // Open the socket connection to receive the data
                    SocketClient = null;
                    SocketClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    SocketClient.ReceiveBufferSize = 1048576;
                    SocketClient.Connect(this.ipTextBox.Text, 8989);

                    // Start the read socket and the user thread
                    g_Client.SingleFramesSize = g_iDataStreamSize;
                    g_Client.NbrOfFrames = Convert.ToInt32(TB_TransferNbr.Text.ToString(), 10);
                    g_Client.Start(SocketClient);
                    ClientThread = new Thread(ClientThreadFct);
                    displayedText = string.Empty;

                    try
                    {
                        //Init and start the sequence acquisition
                        g_RetValue = MSXExxxx.AcquisitionSequenceInitAndStart(ulSequenceSize,
                            g_SequenceChannelList,
                            Convert.ToUInt64(TB_RefreshTime.Text),
                            Convert.ToUInt64(LB_RefreshTimeUnit.SelectedIndex),
                            Convert.ToUInt32(TB_SequenceNbr.Text),
                            Convert.ToUInt32(TB_TransferNbr.Text),
                            Convert.ToUInt32(LB_TriggerMask.SelectedValue),
                            Convert.ToUInt32(LB_TriggerMode.SelectedValue),
                            Convert.ToUInt32(LB_HWEdge.SelectedValue),
                            Convert.ToUInt32(TB_HWTriggerCount.Text),
                            Convert.ToUInt32(TB_SequenceTrigger.Text),
                            ulDataFormat,
                            Convert.ToUInt64(CB_ForceStart.CheckState),
                            0, 0, 0, out g_syserrno);

                        if (g_RetValue != 0)
                        {
                            MessageBox.Show("AcquisitionSequenceInitAndStart Error : " + g_RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                        else
                        {
                            DisplayTimer.Enabled = true;
                            ClientThread.Priority = ThreadPriority.Lowest;
                            ClientThread.Start();
                            B_Connect.Enabled = false;
                            B_StartSample.Enabled = false;
                            GB_SequenceConfiguration.Enabled = false;
                            B_StopSample.Enabled = true;
                        }
                    }

                    catch (Exception ex)
                    {
                        MessageBox.Show("AcquisitionSequenceInitAndStart Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                    
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

                // Test if error occur
                if (B_StartSample.Enabled == true)
                {
                    B_StopSample_Click(sender, e);
                }
            }


        }

        private void B_StopSample_Click(object sender, EventArgs e)
        {

            DisplayTimer.Enabled = false;

            // Test if socket open
            if (SocketClient != null)
            {
                if (g_Client != null)
                    g_Client.Stoop();

                try
                {
                    // Close the socket connection
                    SocketClient.Shutdown(SocketShutdown.Both);
                    SocketClient.Close();
                    SocketClient = null;
                    System.GC.Collect();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket close connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            // Test if sequence started
            if ((B_StartSample.Enabled == false) && (B_StopSample.Enabled == true))
            {

                try
                {
                    // Stop and release the sequence

                    g_RetValue = MSXExxxx.AcquisitionSequenceStopAndRelease(out g_syserrno);

                    if (g_RetValue != 0)
                    {
                        MessageBox.Show("AcquisitionSequenceStopAndRelease Error : " + g_RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("AcquisitionSequenceStopAndRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            B_StartSample.Enabled = true;
            B_StopSample.Enabled = false;
            B_Connect.Enabled = true;
            GB_SequenceConfiguration.Enabled = true;


        }

        /// <summary>
        /// Display timer ticks
        /// Update the value text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            TB_Values.Text = displayedText;
        }

        private void TB_KeyPress_OnlyNumber(object sender, KeyPressEventArgs e)
        {
            if (!char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar))
                e.Handled = true;
        }

        private void LB_Sequence_SelectedIndexChanged(object sender, EventArgs e)
        {
            int i;

            for (i = 0; i < (int)(acquisitionChannelsCount - 1); i++)
            {
                if (g_SequenceChannels[i].SelectedIndex == 0)
                {
                    for (int j = i + 1; j < (int) acquisitionChannelsCount; j++)
                    {
                        if (g_SequenceChannels[j].SelectedIndex != -1)
                        {
                            g_SequenceChannels[j].SelectedIndex = 0;
                            g_SequenceChannels[j].Enabled = false;
                        }
                    }
                    break;
                }
                else if (g_SequenceChannels[i + 1].SelectedIndex != -1)
                {
                    g_SequenceChannels[i + 1].Enabled = true;
                }
            }

            // actualise the sequence size
            for (i = ((int)acquisitionChannelsCount - 1); i >= 0; i--)
            {
                if (g_SequenceChannels[i].Enabled == true)
                {
                    if (g_SequenceChannels[i].SelectedIndex == 0)
                        i = i - 1;
                    break;
                }
            }

            L_SequenceSize.Text = "Sequence size:" + (i + 1).ToString();


        }

        /// <summary>
        /// Form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Sequence_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();

            B_StopSample_Click(sender, null);
        }

        /// <summary>
        /// User choose the absolute time stamp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CB_DataFrameAbsoluteTimeStamp_CheckedChanged(object sender, EventArgs e)
        {
            // We cannot activate both absolute and relative time stamp
            if (CB_DataFrameAbsoluteTimeStamp.Checked)
                CB_DataFrameRelativeTimeStamp.Checked = false;
        }

        /// <summary>
        /// User choose the relative time stamp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CB_DataFrameRelativeTimeStamp_CheckedChanged(object sender, EventArgs e)
        {
            // We cannot activate both absolute and relative time stamp
            if (CB_DataFrameRelativeTimeStamp.Checked)
                CB_DataFrameAbsoluteTimeStamp.Checked = false;
        }

        /// <summary>
        /// User changed the trigger config
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TriggerConfigChanged(object sender, EventArgs e)
        {
            try
            {
                //Trigger mask disabled
                if (LB_TriggerMask.SelectedIndex == 0)
                {
                    LB_TriggerMode.Enabled = false;
                    TB_SequenceTrigger.Enabled = false;
                    LB_HWEdge.Enabled = false;
                    TB_HWTriggerCount.Enabled = false;

                    LB_TriggerMode.SelectedIndex = 0;
                    TB_SequenceTrigger.Text = "1";
                    LB_HWEdge.SelectedIndex = 0;
                    TB_HWTriggerCount.Text = "1";
                }

                //Trigger mask hardware
                if (LB_TriggerMask.SelectedIndex == 1)
                {
                    LB_TriggerMode.Enabled = true;
                    TB_SequenceTrigger.Enabled = true;
                    LB_HWEdge.Enabled = true;
                    TB_HWTriggerCount.Enabled = true;
                }

                //Trigger mask synchro
                if (LB_TriggerMask.SelectedIndex == 2)
                {
                    LB_TriggerMode.Enabled = true;
                    TB_SequenceTrigger.Enabled = true;
                    LB_HWEdge.Enabled = false;
                    TB_HWTriggerCount.Enabled = false;
                    LB_HWEdge.SelectedIndex = 0;
                    TB_HWTriggerCount.Text = "1";
                }
                // Trigger mask compare
                if (LB_TriggerMask.SelectedIndex == 3)
                {
                    LB_TriggerMode.Enabled = true;
                    TB_SequenceTrigger.Enabled = true;
                    LB_HWEdge.Enabled = false;
                    TB_HWTriggerCount.Enabled = false;
                    LB_HWEdge.SelectedIndex = 0;
                    TB_HWTriggerCount.Text = "1";
                }
                // Trigger mask index
                if (LB_TriggerMask.SelectedIndex == 4)
                {
                    LB_TriggerMode.Enabled = true;
                    TB_SequenceTrigger.Enabled = true;
                    LB_HWEdge.Enabled = false;
                    TB_HWTriggerCount.Enabled = false;
                    LB_HWEdge.SelectedIndex = 0;
                    TB_HWTriggerCount.Text = "1";
                }

                //Trigger mode one shot
                if (LB_TriggerMode.SelectedIndex == 0)
                {
                    TB_SequenceTrigger.Enabled = false;
                    TB_SequenceTrigger.Text = "1";
                }

                //Trigger mode sequence
                if (LB_TriggerMode.SelectedIndex == 1)
                {
                    TB_SequenceTrigger.Enabled = true;
                }
            }
            catch { }
        }
    }
}