using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;


namespace Initialisation
{
    public partial class FInitialisation : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXExxxx_WebReference.MSXExxxx MSXExxxx;
        private MXCommon_WebReference.DefaultResponse MXResponse;
        private MSXExxxx_WebReference.DefaultResponse Response;
        private GroupBox[] g_ChannelGroups = new GroupBox[8];
        private CheckBox[] g_Channels = new CheckBox[16];
        private Label[] g_ChannelInfo = new Label[16];
        private ComboBox[] g_ChannelType = new ComboBox[16];
        private TextBox[] g_ChannelTypeValue = new TextBox[16];
        private ComboBox[] g_SamplingRate = new ComboBox[8];
        private ulong g_ulNbrOfChannels = 0;
        private int g_RetValue = 0;
        private int g_syserrno = 0;

        public FInitialisation()
        {
            InitializeComponent();
        }
        public class ListData
        {
            private string myItem;
            private long myValue;

            public ListData(string Item, long Value)
            {

                this.myItem = Item;
                this.myValue = Value;
            }

            public string Item
            {
                get { return myItem; }
            }

            public long Value
            {

                get { return myValue; }
            }
        }


        private void Combobox_set_SelectedValue(ComboBox cb, long value)
        {
            for (int Cpt = 0; Cpt < cb.Items.Count; Cpt++)
            {
                if (Convert.ToInt64(((ListData)cb.Items[Cpt]).Value) == value)
                {
                    cb.SelectedIndex = Cpt;
                    break;
                }
            }

        }

        private void GetConfiguration()
        {
            UInt64 ulClass = 0;
            UInt64 ulType = 0;
            UInt64 ulBaseSamplingRate = 0;
            String[] sClass = { "RTD", "TC", "NTC" };
            

            for (ulong ulCpt = 0; ulCpt < g_ulNbrOfChannels; ulCpt++)
            {

                try
                {
                    Response = MSXExxxx.TemperatureGetConfiguration(ulCpt,0, out ulClass,out ulType,out ulBaseSamplingRate);
                    if (Response.iReturnValue != 0)
                    {
                        MessageBox.Show("TemperatureGetConfiguration  Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                    g_ChannelInfo[ulCpt].Text = sClass[ulClass];

                    ArrayList ALChannelType = new ArrayList();
                    if (sClass[ulClass] == "RTD")
                    {
                        ALChannelType.Add(new ListData("PT", 0));
                        ALChannelType.Add(new ListData("Ni", 1));
                    }
                    else if (sClass[ulClass] == "TC")
                    {
                        ALChannelType.Add(new ListData("B", 1));
                        ALChannelType.Add(new ListData("E", 4));
                        ALChannelType.Add(new ListData("J", 9));
                        ALChannelType.Add(new ListData("K", 10));
                        ALChannelType.Add(new ListData("N", 13));
                        ALChannelType.Add(new ListData("R", 17));
                        ALChannelType.Add(new ListData("S", 18));
                        ALChannelType.Add(new ListData("T", 19));
                    }
                    else if (sClass[ulClass] == "NTC")
                    {
                        ALChannelType.Add(new ListData("G10K4D453", 1));
                    }

                    g_ChannelType[ulCpt].DisplayMember = "Item";
                    g_ChannelType[ulCpt].ValueMember = "Value";
                    g_ChannelType[ulCpt].DataSource = ALChannelType;


                    if (sClass[ulClass] == "RTD")
                    {
                        g_ChannelTypeValue[ulCpt].Visible = true;
                        // test if Ni type
                        if ((ulType & 1) == 1)
                        {
                            g_ChannelTypeValue[ulCpt].Text = (ulType - 1).ToString();
                            Combobox_set_SelectedValue(g_ChannelType[ulCpt], 1);
                        }
                        else
                        {
                            g_ChannelTypeValue[ulCpt].Text = ulType.ToString();
                            Combobox_set_SelectedValue(g_ChannelType[ulCpt],0);
                        }
                    }
                    else
                    {
                        g_ChannelTypeValue[ulCpt].Visible = false;
                        Combobox_set_SelectedValue(g_ChannelType[ulCpt],(long)ulType);
                    }

                    ArrayList ALSamplingRate = new ArrayList();
                    ALSamplingRate.Add(new ListData("5 Hz", 5));
                    ALSamplingRate.Add(new ListData("10 Hz", 10));
                    ALSamplingRate.Add(new ListData("20 Hz", 20));
                    ALSamplingRate.Add(new ListData("40 Hz", 40));
                    ALSamplingRate.Add(new ListData("80 Hz", 80));
                    ALSamplingRate.Add(new ListData("160 Hz", 160));
                    ALSamplingRate.Add(new ListData("320 Hz", 320));
                    ALSamplingRate.Add(new ListData("640 Hz", 640));
                    ALSamplingRate.Add(new ListData("1000 Hz", 1000));
                    ALSamplingRate.Add(new ListData("2000 Hz", 2000));

                    g_SamplingRate[ulCpt / 2].DisplayMember = "Item";
                    g_SamplingRate[ulCpt / 2].ValueMember = "Value";
                    g_SamplingRate[ulCpt / 2].DataSource = ALSamplingRate;

                    Combobox_set_SelectedValue(g_SamplingRate[ulCpt / 2],(long)ulBaseSamplingRate);


                }
                catch (Exception ex)
                {
                    MessageBox.Show("TemperatureGetConfiguration  Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

            }


        }

                
        private void SetConfiguration()
        {
            UInt64 ulType = 0;


            for (ulong ulCpt = 0; ulCpt < g_ulNbrOfChannels; ulCpt++)
            {
                if (g_Channels[ulCpt].Enabled == true)
                {
                    try
                    {
                        g_RetValue = MSXExxxx.TemperatureSetSamplingRate(ulCpt / 2, Convert.ToUInt64(g_SamplingRate[ulCpt / 2].SelectedValue), 0, out g_syserrno);
                        if (g_RetValue != 0)
                        {
                            MessageBox.Show("TemperatureSetSamplingRate  Error : " + g_RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            return;
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("TemperatureSetSamplingRate  Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }

                    try
                    {
                        if (g_ChannelInfo[ulCpt].Text == "RTD")
                            ulType = Convert.ToUInt64(g_ChannelTypeValue[ulCpt].Text) + Convert.ToUInt64(g_ChannelType[ulCpt].SelectedValue);
                        else
                            ulType = Convert.ToUInt64(g_ChannelType[ulCpt].SelectedValue);

                        g_RetValue = MSXExxxx.TemperatureSetChannelType(ulCpt, ulType, 0, out g_syserrno);
                        if (g_RetValue != 0)
                        {
                            MessageBox.Show("TemperatureSetChannelType  Error : " + g_RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            return;
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("TemperatureSetChannelType  Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                }
            }

        }
        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];


            if ((TB_IPAddress_0.Text.ToString() == "") || (TB_IPAddress_1.Text.ToString() == "") || (TB_IPAddress_2.Text.ToString() == "") || (TB_IPAddress_3.Text.ToString() == ""))
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                if ((Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256))
                {
                    if ((Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256))
                    {
                        if ((Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256))
                        {
                            if ((Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256))
                            {
                                B_SetConfiguration.Enabled = false;
                                MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
                                MSXExxxx.Url = MX_Common.Url;

                                // Get the module type
                                try
                                {
                                    MXResponse = MX_Common.GetModuleType(out pcName);
                                    if (MXResponse.iReturnValue != 0)
                                    {
                                        MessageBox.Show("GetModuleType Error : " + MXResponse.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }

                                }
                                catch (Exception ex)
                                {
                                    MessageBox.Show("GetModuleType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }


                                try
                                {
                                    Response = MSXExxxx.TemperatureGetNumberOfChannels(0, out g_ulNbrOfChannels);
                                    if (Response.iReturnValue != 0)
                                    {
                                        MessageBox.Show("TemperatureGetNumberOfChannels Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }

                                }
                                catch (Exception ex)
                                {
                                    MessageBox.Show("AcquisitionGetNumberOfChannels Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }

                                for (ulong i = 0; i < g_ulNbrOfChannels; i++)
                                {
                                    g_Channels[i].Enabled = true;
                                    g_Channels[i].Checked = true;
                                    g_ChannelGroups[i / 2].Enabled = true;
                                }

                                for (ulong i = g_ulNbrOfChannels; i < 16; i++)
                                {
                                    g_Channels[i].Enabled = false;
                                    g_Channels[i].Checked = false;
                                    g_ChannelGroups[i / 2].Enabled = false;
                                }

                                GetConfiguration();
                                B_SetConfiguration.Enabled = true;
                                GB_ChannelConfiguration.Enabled = true;
                            }
                            else
                            {
                                MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                        else
                        {
                            MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

        }

        private void FInitialisation_Load(object sender, EventArgs e)
        {
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXExxxx = new MSXExxxx_WebReference.MSXExxxx();
            MX_Common.Timeout = 5000;
            MSXExxxx.Timeout = 5000;

            // Save the auto refresh channels check box
            g_Channels[0] = CB_Channel0;
            g_Channels[1] = CB_Channel1;
            g_Channels[2] = CB_Channel2;
            g_Channels[3] = CB_Channel3;
            g_Channels[4] = CB_Channel4;
            g_Channels[5] = CB_Channel5;
            g_Channels[6] = CB_Channel6;
            g_Channels[7] = CB_Channel7;
            g_Channels[8] = CB_Channel8;
            g_Channels[9] = CB_Channel9;
            g_Channels[10] = CB_Channel10;
            g_Channels[11] = CB_Channel11;
            g_Channels[12] = CB_Channel12;
            g_Channels[13] = CB_Channel13;
            g_Channels[14] = CB_Channel14;
            g_Channels[15] = CB_Channel15;


            g_ChannelGroups[0] = GB_Group0;
            g_ChannelGroups[1] = GB_Group1;
            g_ChannelGroups[2] = GB_Group2;
            g_ChannelGroups[3] = GB_Group3;
            g_ChannelGroups[4] = GB_Group4;
            g_ChannelGroups[5] = GB_Group5;
            g_ChannelGroups[6] = GB_Group6;
            g_ChannelGroups[7] = GB_Group7;

            g_ChannelInfo[0] = L_TypeCh0;
            g_ChannelInfo[1] = L_TypeCh1;
            g_ChannelInfo[2] = L_TypeCh2;
            g_ChannelInfo[3] = L_TypeCh3;
            g_ChannelInfo[4] = L_TypeCh4;
            g_ChannelInfo[5] = L_TypeCh5;
            g_ChannelInfo[6] = L_TypeCh6;
            g_ChannelInfo[7] = L_TypeCh7;
            g_ChannelInfo[8] = L_TypeCh8;
            g_ChannelInfo[9] = L_TypeCh9;
            g_ChannelInfo[10] = L_TypeCh10;
            g_ChannelInfo[11] = L_TypeCh11;
            g_ChannelInfo[12] = L_TypeCh12;
            g_ChannelInfo[13] = L_TypeCh13;
            g_ChannelInfo[14] = L_TypeCh14;
            g_ChannelInfo[15] = L_TypeCh15;

            g_ChannelType[0] = CB_Type0;
            g_ChannelType[1] = CB_Type1;
            g_ChannelType[2] = CB_Type2;
            g_ChannelType[3] = CB_Type3;
            g_ChannelType[4] = CB_Type4;
            g_ChannelType[5] = CB_Type5;
            g_ChannelType[6] = CB_Type6;
            g_ChannelType[7] = CB_Type7;
            g_ChannelType[8] = CB_Type8;
            g_ChannelType[9] = CB_Type9;
            g_ChannelType[10] = CB_Type10;
            g_ChannelType[11] = CB_Type11;
            g_ChannelType[12] = CB_Type12;
            g_ChannelType[13] = CB_Type13;
            g_ChannelType[14] = CB_Type14;
            g_ChannelType[15] = CB_Type15;

            g_ChannelTypeValue[0] = TB_TypeValue0;
            g_ChannelTypeValue[1] = TB_TypeValue1;
            g_ChannelTypeValue[2] = TB_TypeValue2;
            g_ChannelTypeValue[3] = TB_TypeValue3;
            g_ChannelTypeValue[4] = TB_TypeValue4;
            g_ChannelTypeValue[5] = TB_TypeValue5;
            g_ChannelTypeValue[6] = TB_TypeValue6;
            g_ChannelTypeValue[7] = TB_TypeValue7;
            g_ChannelTypeValue[8] = TB_TypeValue8;
            g_ChannelTypeValue[9] = TB_TypeValue9;
            g_ChannelTypeValue[10] = TB_TypeValue10;
            g_ChannelTypeValue[11] = TB_TypeValue11;
            g_ChannelTypeValue[12] = TB_TypeValue12;
            g_ChannelTypeValue[13] = TB_TypeValue13;
            g_ChannelTypeValue[14] = TB_TypeValue14;
            g_ChannelTypeValue[15] = TB_TypeValue15;

            g_SamplingRate[0] = CB_SamplingRateGroup0;
            g_SamplingRate[1] = CB_SamplingRateGroup1;
            g_SamplingRate[2] = CB_SamplingRateGroup2;
            g_SamplingRate[3] = CB_SamplingRateGroup3;
            g_SamplingRate[4] = CB_SamplingRateGroup4;
            g_SamplingRate[5] = CB_SamplingRateGroup5;
            g_SamplingRate[6] = CB_SamplingRateGroup6;
            g_SamplingRate[7] = CB_SamplingRateGroup7;


            GB_ChannelConfiguration.Enabled = false;
            B_SetConfiguration.Enabled = false;

        }

        private void B_SetConfiguration_Click(object sender, EventArgs e)
        {
            SetConfiguration();
        }


        private void TB_KeyPress_OnlyNumber(object sender, KeyPressEventArgs e)
        {
            if (!char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar))
                e.Handled = true;
        }


    }
}