using System;
using System.Collections.Generic;
using System.Text;

namespace Database
{
    public class MSXExxxx
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXExxxx(String ip)
        {
            // Construct the web reference
            this.sys = new MSXExxxxWeb.MSXExxxx();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 10000; // Database accesses need more time
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the transducer database from the system
        /// </summary>
        /// <param name="items"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerDatabaseGetList(ref List<TransducerDatabaseItem> items, ref string error)
        {
            // initialisation
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            // Get the list
            try
            {
                MSXExxxxWeb.TransducerTypeInfo[] info = null;
                ulong transducerCount = 0;
                MSXExxxxWeb.DefaultResponse response = this.sys.TransducerDataBaseGetList(0, out transducerCount, out info);
                if (response.iReturnValue != 0)
                {
                    error = "TransducerDataBaseGetList error: " + response.iReturnValue.ToString();
                    return -2;
                }

                // Loop through the transducers
                for (ulong i = 0; i < transducerCount; i++)
                {
                    string name = new ASCIIEncoding().GetString(info[i].cName);
                    items.Add(new TransducerDatabaseItem(info[i].ulTransducerIndex,
                        name,
                        info[i].ulType,
                        info[i].dRange,
                        info[i].ulNominalFrequency,
                        info[i].dVeff,
                        info[i].dSensitivity,
                        info[i].ulImpedance));
                }
            }
            catch (Exception ex)
            {
                error = "TransducerDataBaseGetList exception: " + ex.Message;
                return -3;
            }

            return 0;
        }

        /// <summary>
        /// Add a transducer into the database
        /// </summary>
        /// <param name="item"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerDatabaseAdd(TransducerDatabaseItem item, ref string error)
        {
            // initialisation
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            try
            {
                int syserrno = 0;
                int retval = this.sys.TransducerDataBaseAdd(item.index,
                    item.name,
                    item.type,
                    item.veff,
                    item.range,
                    item.sensitivity,
                    item.frequency,
                    item.impedance,
                    0,
                    out syserrno);
                if (retval != 0)
                {
                    error = "TransducerDataBaseAdd error: " + retval.ToString();
                    return -2;
                }
            }
            catch (Exception ex)
            {
                error = "TransducerDataBaseAdd exception: " + ex.Message;
                return -3;
            }

            return 0;
        }

        /// <summary>
        /// Delete a transducer from the database
        /// </summary>
        /// <param name="index"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerDatabaseDelete(ulong index, ref string error)
        {
            // initialisation
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            try
            {
                int syserrno = 0;
                int retval = this.sys.TransducerDataBaseDelete(index, 0, out syserrno);
                if (retval != 0)
                {
                    error = "TransducerDataBaseDelete error: " + retval.ToString();
                    return -2;
                }
            }
            catch (Exception ex)
            {
                error = "TransducerDataBaseDelete exception: " + ex.Message;
                return -3;
            }

            return 0;
        }
    }
}
