using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Database
{
    public class TransducerDatabaseItem
    {
        public ulong index;
        public string name;
        public ulong type; // 0: HB, 1: LVDT, 2: MAHR-HB, 3: MAHR-LVDT
        public double range;
        public ulong frequency;
        public double veff;
        public double sensitivity;
        public ulong impedance;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="index"></param>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <param name="range"></param>
        /// <param name="frequency"></param>
        /// <param name="veff"></param>
        /// <param name="sensitivty"></param>
        /// <param name="impedance"></param>
        public TransducerDatabaseItem(ulong index, string name, ulong type, double range, ulong frequency, double veff, double sensitivity, ulong impedance)
        {
            this.index = index;
            this.name = name;
            this.type = type;
            this.range = range;
            this.frequency = frequency;
            this.veff = veff;
            this.sensitivity = sensitivity;
            this.impedance = impedance;
        }

        private string GetTypeAsString(ulong type)
        {
            switch (type)
            {
                case 0:
                    return "HB";
                case 1:
                    return "LVDT";
                case 2:
                    return "MAHR-HB";
                case 3:
                    return "MAHR-LVDT";
                default:
                    return type.ToString();
            }
        }

        /// <summary>
        /// Make the construction of the list view item easiest
        /// </summary>
        /// <returns></returns>
        public ListViewItem ConstructListViewItem()
        {
            return new ListViewItem(new string[] { this.index.ToString(),
                this.name,
                this.GetTypeAsString(this.type),
                this.range.ToString(),
                this.frequency.ToString(),
                this.veff.ToString(),
                this.sensitivity.ToString(),
                this.impedance.ToString()});
        }
    }
}
