using System;
using System.Collections.Generic;
using System.Text;

namespace TransducerCalibrationData
{
    public class MSXExxxx
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXExxxx(String ip)
        {
            // Construct the web reference
            this.sys = new MSXExxxxWeb.MSXExxxx();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 5000;
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the number of transducer channels
        /// </summary>
        /// <param name="channelNb"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerGetNumberOfChannels(ref ulong channelNb, ref string error)
        {
            // init
            error = string.Empty;
            channelNb = 0;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.TransducerGetNumberOfChannels(0, out channelNb);
                if (response.iReturnValue != 0)
                {
                    error = "TransducerGetNumberOfChannels error: " + response.iReturnValue.ToString();
                    return -2;
                }
            }
            catch (Exception ex)
            {
                error = "TransducerGetNumberOfChannels exception: " + ex.Message;
                return -3;
            }

            return 0;
        }

        /// <summary>
        /// Get the calibration data for the selected channel
        /// </summary>
        /// <param name="channel"></param>
        /// <param name="calibrationData"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int TransducerGetChannelTransducersCalibrationDataList(ulong channel, ref List<TransducerCalibration> calibrationData, ref string error)
        {
            // init
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -1;
            }

            ulong count = 0;
            MSXExxxxWeb.TransducerCalibrationInfo [] info = null;

            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.TransducerGetChannelTransducersCalibrationDataList(channel, 0,
                    out count,
                    out info);
                if (response.iReturnValue != 0)
                {
                    error = "TransducerGetChannelTransducersCalibrationDataList error: " + response.iReturnValue.ToString();
                    return -2;
                }

                // Construct the outside list
                calibrationData = new List<TransducerCalibration>();
                for (ulong i = 0; i < count; i++)
                {
                    calibrationData.Add(new TransducerCalibration(info[i].ulUsed,
                        info[i].ulCalibrated,
                        info[i].ulTransducerIndex,
                        info[i].ulSecGainCoarse,
                        info[i].ulSecGainFine,
                        info[i].ulFeedbackGainCoarse,
                        info[i].ulFeedbackGainFine,
                        info[i].ulFeedbackOffset,
                        info[i].ulCorrectionFactorLeft,
                        info[i].ulCorrectionFactorRight));
                }
            }
            catch (Exception ex)
            {
                error = "TransducerGetChannelTransducersCalibrationDataList exception: " + ex.Message;
                return -3;
            }

            return 0;
        }
    }
}
